%define anolis_release 2
%define rpmhome /usr/lib/rpm
%bcond_with check
%bcond_without xz
%bcond_without plugins
%bcond_without libarchive
%bcond_without libimaevm
%bcond_with fsverity
%bcond_without zstd
%bcond_without ndb
%bcond_without sqlite
%bcond_without bdb_ro

%global rpmver 4.18.1
%global sover 9

%global srcver %{rpmver}
%global srcdir rpm-%(echo %{rpmver} | cut -d'.' -f1-2).x

Summary: The RPM package management system
Name: rpm
Version: %{rpmver}
Release: %{anolis_release}%{?dist}
Url: http://www.rpm.org/
Source0: http://ftp.rpm.org/releases/%{srcdir}/rpm-%{srcver}.tar.bz2

Source10: rpmdb-rebuild.service

Source20: rpmdb-migrate.service
Source21: rpmdb_migrate

Patch0: rpm-4.17.x-rpm_dbpath.patch
# Promote RPM_LDFLAGS to environment
Patch1: rpm-4.18.x-ldflags.patch
# Allow install docs when building without pandoc
Patch2: rpm-4.18.x-revert-pandoc-cond.patch
# Deal segfault with sqlite >= 3.42.0
# https://github.com/rpm-software-management/rpm/commit/ea3187cfcf9cac87e5bc5e7db79b0338da9e355e
Patch3: rpm-4.18.1-Dont-muck-with-per-process-global-sqlite-configuration.patch
License: GPLv2+

BuildRequires: automake libtool system-rpm-config systemd-rpm-macros make gawk
BuildRequires: elfutils-devel >= 0.112 elfutils-libelf-devel
BuildRequires: readline-devel zlib-devel openssl-devel ncurses-devel
BuildRequires: popt-devel >= 1.10.2 file-devel gettext-devel libacl-devel
BuildRequires: bzip2-devel >= 0.9.0c pkgconfig(lua) >= 5.2 libcap-devel 

Requires: coreutils
Requires: popt >= 1.10.2.1
Requires: curl
Obsoletes: python2-rpm < %{version}-%{release}

%if %{with check}
BuildRequires: fakechroot gnupg2 debugedit >= 0.3
%endif

%if %{with plugins}
BuildRequires: libselinux-devel dbus-devel audit-libs-devel
%endif

%if %{with libimaevm}
BuildRequires: ima-evm-utils-devel >= 1.0
%endif

%if %{with fsverity}
BuildRequires: fsverity-utils-devel
%endif

%if %{with sqlite}
BuildRequires: sqlite-devel
%endif

%if %{with zstd}
BuildRequires: libzstd-devel
%endif

%if %{with libarchive}
BuildRequires: libarchive-devel
%endif

%if %{with xz}
BuildRequires: xz-devel >= 4.999.8
%endif

Requires(pre): coreutils findutils sed

%description
RPM Package Manager (RPM) is a free and open-source tool that enables you to 
install, remove, update, verify and manage software packages on Linux-based 
systems. RPM packages are files that contain the compiled version of a 
software project, along with metadata such as its name, version, description 
and dependencies. You can also create and distribute source RPM packages 
(SRPMs), which contain the source code and instructions for building a binary 
RPM package.

%package libs
Summary:  Libraries for manipulating RPM packages
License: GPLv2+ and LGPLv2+ with exceptions
Requires: %{name} = %{version}-%{release}

%description libs
This package contains the RPM shared libraries. 

The RPM shared libraries are used by dynamic executables 
that depend on them at run time. They can be installed in different locations 
and configured with ldconfig or rpath . The RPM shared libraries have different 
versions and symbols that are tracked by RPM using the soname and unique 
identifiers . This package allows you to build packages that are compatible 
with the RPM shared libraries on your system.

%package build-libs
Summary:  Libraries for building RPM packages
License: GPLv2+ and LGPLv2+ with exceptions
Requires: rpm-libs = %{version}-%{release}

%description build-libs
This package provides the RPM shared libraries that are needed for creating 
packages with RPM. 

%package sign-libs
Summary:  Libraries for signing RPM packages
License: GPLv2+ and LGPLv2+ with exceptions
Requires: rpm-libs = %{version}-%{release}
Requires: %{_bindir}/gpg2

%description sign-libs
This package provides the RPM shared libraries for package signing. Package 
signing is a process of adding a digital signature to an RPM package to 
verify its integrity and authenticity. The rpm utility uses GPG keys to sign 
and verify packages. The sign-libs package contains the libraries that 
implement the signing and verification algorithms for RPM. This package is 
required for building and installing signed RPM packages.

%package devel
Summary:  Development files for manipulating RPM packages
License: GPLv2+ and LGPLv2+ with exceptions
Requires: %{name} = %{version}-%{release}
Requires: %{name}-libs = %{version}-%{release}
Requires: %{name}-build-libs = %{version}-%{release}
Requires: %{name}-sign-libs = %{version}-%{release}
Requires: popt-devel

%description devel
This package includes the C library and header files for RPM. These development 
files allow you to write programs that can manipulate RPM packages and 
databases. These files are designed to make it easier to create graphical 
package managers or any other tools that require a deep understanding of 
RPM packages to function.

This package should be installed if you want to develop programs that
will manipulate RPM packages and databases.

%package build
Summary: Scripts and executable programs used to build packages
Requires: rpm = %{version}-%{release}
Requires: elfutils >= 0.128 binutils
Requires: findutils sed grep gawk diffutils file patch >= 2.5
Requires: tar unzip gzip bzip2 cpio xz
%if %{with zstd}
Requires: zstd
%endif
Requires: debugedit >= 0.3
Requires: pkgconfig >= 1:0.24
Requires: /usr/bin/gdb-add-index
Suggests: gdb-minimal
Requires: system-rpm-config

%description build
The rpm-build package enables you to create binary and source packages using 
the RPM Package Manager. A package is a collection of files and meta-data that 
can be installed and removed from a system. The meta-data contains information 
such as scripts, attributes, and descriptions of the package. The rpm-build 
package provides the tools and scripts that are required to build packages 
from source code or pre-compiled binaries. You can use the rpm-build package 
to create custom packages for your own software or modify existing packages 
to suit your needs.

%package sign
Summary: Package signing support
Requires: rpm-sign-libs = %{version}-%{release}

%description sign
This package contains support for digitally signing RPM packages.

%package -n python3-%{name}
Summary: Python 3 bindings for apps which will manipulate RPM packages
BuildRequires: python3-devel
%{?python_provide:%python_provide python3-%{name}}
Requires: %{name}-libs = %{version}-%{release}
Provides: %{name}-python3 = %{version}-%{release}
Obsoletes: %{name}-python3 < %{version}-%{release}
Obsoletes: platform-python-%{name} < %{version}-%{release}

%description -n python3-%{name}
python3-rpm provides a module that allows you to use the interface provided by 
RPM libraries in your Python 3 code. You can access various functions and 
classes related to RPM packages and databases using this module.
 
If you are developing Python 3 applications that need to interact with RPM 
Package Manager libraries, you should install the python3-rpm package.

%package apidocs
Summary: API documentation for RPM libraries
BuildArch: noarch

%description apidocs
This package provides API documentation for RPM libraries that can be used to 
create applications that work with RPM packages and databases. The documentation 
covers various aspects of RPM development, such as package building, dependency 
resolution, signature verification, and transaction management. The documentation 
is generated using Doxygen and can be browsed online or offline. The package is 
useful for developers who want to learn more about the RPM internals and APIs.

%package cron
Summary: Create daily logs of installed packages.
BuildArch: noarch
Requires: crontabs logrotate rpm = %{version}-%{release}

%description cron
rpm-cron is a package that contains a cron job for logging the packages 
installed on a system on a daily basis. Cron is a service that allows you 
to schedule tasks to run at regular intervals. The rpm-cron package is useful 
for keeping track of the changes in the system's package inventory. 

%if %{with plugins}
%package plugin-selinux
Summary: Rpm plugin for SELinux functionality
Requires: rpm-libs = %{version}-%{release}
Requires: selinux-policy-base

%description plugin-selinux
%{summary}.

%package plugin-syslog
Summary: Rpm plugin for syslog functionality
Requires: rpm-libs = %{version}-%{release}

%description plugin-syslog
%{summary}.

%package plugin-systemd-inhibit
Summary: Rpm plugin for systemd inhibit functionality
Requires: rpm-libs = %{version}-%{release}

%description plugin-systemd-inhibit
This plugin for RPM uses the systemd-inhibit mechanism to block the system from 
going into idle, sleep or shutdown mode while an rpm transaction is in progress. 
This prevents possible system damage that could result from a transaction being 
aborted by a reboot.
For more details on how this mechanism works, see systemd-inhibit(1). It is 
highly recommended to install this plugin on all systems that use systemd as 
the init system and have the DBUS system bus available. If the plugin cannot 
access the inhibit interface, it will issue a warning but will not stop the 
transaction. There are no specific options for this plugin. See rpm-plugins(8) 
for general information on how to control plugins.

%package plugin-ima
Summary: Rpm plugin ima file signatures
Requires: rpm-libs = %{version}-%{release}

%description plugin-ima
%{summary}.

%package plugin-prioreset
Summary: Rpm plugin for resetting scriptlet priorities for SysV init
Requires: rpm-libs = %{version}-%{release}

%description plugin-prioreset
%{summary}.

Useful on legacy SysV init systems if you run rpm transactions with
nice/ionice priorities. Should not be used on systemd systems.

%package plugin-audit
Summary: Rpm plugin for logging audit events on package operations
Requires: rpm-libs = %{version}-%{release}

%description plugin-audit
%{summary}.

%package plugin-fsverity
Summary: Rpm plugin for fsverity file signatures
Requires: rpm-libs = %{version}-%{release}

%description plugin-fsverity
%{summary}.

%package plugin-fapolicyd
Summary: Rpm plugin for fapolicyd support
Requires: rpm-libs = %{version}-%{release}
Provides: fapolicyd-plugin = %{version}-%{release}
Obsoletes: fapolicyd-dnf-plugin < 1.0.5

%description plugin-fapolicyd
%{summary}.

%package plugin-dbus-announce
Summary: Rpm plugin for announcing transactions on the DBUS
Requires: rpm-libs = %{version}-%{release}

%description plugin-dbus-announce
This plugin broadcasts basic information about rpm transactions to the system 
DBUS, such as which packages are being installed or removed. This way, other 
applications can listen to the signals and get updated on the changes in the 
system packages. This plugin is useful for open source software development 
and testing on Linux operating systems.

%endif

%prep
%autosetup -n rpm-%{srcver} -p1

%build
%set_build_flags

autoreconf -i -f

for i in $(find . -name ltmain.sh) ; do
     %{__sed} -i.backup -e 's~compiler_flags=$~compiler_flags="%{_hardened_ldflags}"~' $i
done;

[ -x /usr/lib/rpm/anolis/force-as-needed-for-shared-lib-in-libtool ] &&  /usr/lib/rpm/anolis/force-as-needed-for-shared-lib-in-libtool .
[ -x /usr/lib/rpm/anolis/fix-libtool-ltmain-from-overlinking ] && /usr/lib/rpm/anolis/fix-libtool-ltmain-from-overlinking
[ -x /usr/lib/rpm/anolis/fix-libtool-from-moving-options-after-libs ] && /usr/lib/rpm/anolis/fix-libtool-from-moving-options-after-libs .
[ -x /usr/lib/rpm/anolis/fix-dlsearch-path-in-libtool ] && /usr/lib/rpm/anolis/fix-dlsearch-path-in-libtool . %{_lib}

./configure \
    --with-crypto=openssl \
    --enable-python \
    %{?with_bdb_ro: --enable-bdb-ro} \
    %{?with_sqlite: --enable-sqlite} \
    %{?with_zstd: --enable-zstd} \
    %{?with_fsverity: --with-fsverity} \
    %{?with_libimaevm: --with-imaevm} \
    %{?with_ndb: --enable-ndb} \
    --with-fapolicyd \
    --with-acl \
    --with-cap \
    --with-selinux \
    %{!?with_plugins: --disable-plugins} \
    --with-vendor=anolis \
    --host=%{_target_platform} \
    --build=%{_target_platform} \
    --libdir=%{_libdir} \
    --sharedstatedir=%{_var}/lib \
    --localstatedir=%{_var} \
    --sysconfdir=%{_sysconfdir} \
    --prefix=%{_usr}

%make_build

%install
%make_install

mkdir -p $RPM_BUILD_ROOT%{_unitdir}
install -m 644 %{SOURCE10} $RPM_BUILD_ROOT/%{_unitdir}
install -m 644 %{SOURCE20} $RPM_BUILD_ROOT/%{_unitdir}

mkdir -p $RPM_BUILD_ROOT%{rpmhome}
install -m 755 %{SOURCE21} $RPM_BUILD_ROOT/%{rpmhome}

mkdir -p ${RPM_BUILD_ROOT}%{_sysconfdir}/cron.daily
install -m 755 scripts/rpm.daily ${RPM_BUILD_ROOT}%{_sysconfdir}/cron.daily/rpm

mkdir -p ${RPM_BUILD_ROOT}%{_sysconfdir}/logrotate.d
install -m 644 scripts/rpm.log ${RPM_BUILD_ROOT}%{_sysconfdir}/logrotate.d/rpm

mkdir -p $RPM_BUILD_ROOT%{_sysconfdir}/rpm
mkdir -p $RPM_BUILD_ROOT%{rpmhome}/macros.d
mkdir -p $RPM_BUILD_ROOT/usr/lib/sysimage/rpm

for be in %{?with_ndb:ndb} %{?with_sqlite:sqlite}; do
    ./rpmdb --define "_db_backend ${be}" --dbpath=${PWD}/${be} --initdb
    cp -va ${be}/. $RPM_BUILD_ROOT/usr/lib/sysimage/rpm/
done

ln -s ../../bin/find-debuginfo $RPM_BUILD_ROOT/usr/lib/rpm/find-debuginfo.sh

%find_lang rpm

rm -f $RPM_BUILD_ROOT/%{rpmhome}/{perldeps.pl,perl.*,pythond*}
rm -f $RPM_BUILD_ROOT/%{_fileattrsdir}/{perl*,python*}
rm -rf $RPM_BUILD_ROOT/var/tmp

%generate_compatibility_deps

%if %{with check}
%check
make check TESTSUITEFLAGS=-j%{_smp_build_ncpus} || (cat tests/rpmtests.log; exit 1)
make clean
%endif

%pre
if [ -d /var/lib/rpm ]; then
    mkdir -p /usr/lib/sysimage/rpm
    rpmdb_files=$(find /var/lib/rpm -maxdepth 1 -type f | sed 's|^/var/lib/rpm/||g' | sort)
    for rpmdb_file in ${rpmdb_files[@]}; do
        ln -sfr /var/lib/rpm/${rpmdb_file} /usr/lib/sysimage/rpm/${rpmdb_file}
    done
fi

%triggerun -- rpm < 4.17.0
if [ -x /usr/bin/systemctl ]; then
    systemctl --no-reload preset rpmdb-migrate ||:
fi

%posttrans
if [ -d /var/lib/rpm ]; then
    touch /var/lib/rpm/.migratedb
fi

%files -f rpm.lang
%license COPYING
%doc CREDITS docs/manual/[a-z]*

%exclude %{abidir}/rpmsign-option.list
%exclude %{abidir}/rpmbuild-option.list
%exclude %{abidir}/rpmspec-option.list
%exclude %{abidir}/rpmlua-option.list

%{_unitdir}/rpmdb-rebuild.service
%{_unitdir}/rpmdb-migrate.service

%dir %{_sysconfdir}/rpm

%attr(0755, root, root) %dir /usr/lib/sysimage/rpm
%attr(0644, root, root) %ghost %config(missingok,noreplace) /usr/lib/sysimage/rpm/*
%attr(0644, root, root) %ghost /usr/lib/sysimage/rpm/.*.lock

%{_bindir}/rpmverify
%{_bindir}/rpmquery
%{_bindir}/rpmkeys
%{_bindir}/rpmdb
%{_bindir}/rpm2cpio
%{_bindir}/rpm2archive
%{_bindir}/rpm
%{abidir}/rpm*-option.list

%attr(0755, root, root) %dir %{rpmhome}
%{rpmhome}/tgpg
%{rpmhome}/rpm2cpio.sh
%{rpmhome}/rpm.supp
%{rpmhome}/rpm.log
%{rpmhome}/rpm.daily
%{rpmhome}/rpmdb_*

%{rpmhome}/rpmrc
%{rpmhome}/rpmpopt*
%{rpmhome}/lua
%{rpmhome}/macros.d
%{rpmhome}/macros

%{rpmhome}/platform

%dir %{rpmhome}/fileattrs

%{_mandir}/man8/rpm-plugins.8*
%{_mandir}/man8/rpm-misc.8*
%{_mandir}/man8/rpm2cpio.8*
%{_mandir}/man8/rpm2archive.8*
%{_mandir}/man8/rpmkeys.8*
%{_mandir}/man8/rpmdb.8*
%{_mandir}/man8/rpm.8*

# XXX this places translated manuals to wrong package wrt eg rpmbuild
%lang(sk) %{_mandir}/sk/man[18]/*.[18]*
%lang(ru) %{_mandir}/ru/man[18]/*.[18]*
%lang(pl) %{_mandir}/pl/man[18]/*.[18]*
%lang(ja) %{_mandir}/ja/man[18]/*.[18]*
%lang(ko) %{_mandir}/ko/man[18]/*.[18]*
%lang(fr) %{_mandir}/fr/man[18]/*.[18]*

%{abidir}/_rpm*.dump


%files libs
%dir %{abidir}
%{_libdir}/librpmio.so.%{sover}
%{_libdir}/librpm.so.%{sover}
%{_libdir}/librpmio.so.%{sover}.*
%{_libdir}/librpm.so.%{sover}.*
%{abidir}/librpm*.dump
%exclude %{abidir}/librpmbuild*.dump
%exclude %{abidir}/librpmsign*.dump
%if %{with plugins}
%dir %{_libdir}/rpm-plugins

%files plugin-syslog
%{_libdir}/rpm-plugins/syslog.so
%{abidir}/syslog.dump
%{_mandir}/man8/rpm-plugin-syslog.8*

%files plugin-selinux
%{_libdir}/rpm-plugins/selinux.so
%{abidir}/selinux.dump
%{_mandir}/man8/rpm-plugin-selinux.8*

%files plugin-systemd-inhibit
%{_libdir}/rpm-plugins/systemd_inhibit.so
%{abidir}/systemd_inhibit.dump
%{_mandir}/man8/rpm-plugin-systemd-inhibit.8*

%files plugin-ima
%{_libdir}/rpm-plugins/ima.so
%{abidir}/ima.dump
%{_mandir}/man8/rpm-plugin-ima.8*

%files plugin-fsverity
%{_libdir}/rpm-plugins/fsverity.so
%{abidir}/fsverity.dump

%files plugin-fapolicyd
%{_libdir}/rpm-plugins/fapolicyd.so
%{abidir}/fapolicyd.dump
%{_mandir}/man8/rpm-plugin-fapolicyd.8*

%files plugin-prioreset
%{_libdir}/rpm-plugins/prioreset.so
%{abidir}/prioreset.dump
%{_mandir}/man8/rpm-plugin-prioreset.8*

%files plugin-audit
%{_libdir}/rpm-plugins/audit.so
%{abidir}/audit.dump
%{_mandir}/man8/rpm-plugin-audit.8*

%files plugin-dbus-announce
%{_libdir}/rpm-plugins/dbus_announce.so
%{abidir}/dbus_announce.dump
%{_mandir}/man8/rpm-plugin-dbus-announce.8*
%{_sysconfdir}/dbus-1/system.d/org.rpm.conf
%endif

%files build-libs
%{_libdir}/librpmbuild.so.%{sover}
%{_libdir}/librpmbuild.so.%{sover}.*
%{abidir}/librpmbuild*.dump

%files sign-libs
%{_libdir}/librpmsign.so.%{sover}
%{_libdir}/librpmsign.so.%{sover}.*
%{abidir}/librpmsign*.dump

%files build
%{rpmhome}/rpmuncompress
%{rpmhome}/find-debuginfo.sh
%{rpmhome}/fileattrs/*
%{rpmhome}/mkinstalldirs
%{rpmhome}/*.req
%{rpmhome}/*.prov
%{rpmhome}/*deps*
%{rpmhome}/*requires*
%{rpmhome}/*provides*
%{rpmhome}/find-lang.sh
%{rpmhome}/check-*
%{rpmhome}/brp-*

%{_bindir}/rpmbuild
%{_bindir}/gendiff
%{_bindir}/rpmspec
%{_bindir}/rpmlua
%{abidir}/rpmbuild-option.list
%{abidir}/rpmspec-option.list
%{abidir}/rpmlua-option.list

%{_mandir}/man1/gendiff.1*
%{_mandir}/man8/rpmbuild.8*
%{_mandir}/man8/rpmdeps.8*
%{_mandir}/man8/rpmspec.8*
%{_mandir}/man8/rpmlua.8*

%files sign
%{_bindir}/rpmsign
%{abidir}/rpmsign-option.list
%{_mandir}/man8/rpmsign.8*

%files -n python3-%{name}
%{python3_sitearch}/rpm/
%{python3_sitearch}/rpm-%{rpmver}*.egg-info

%files devel
%{_includedir}/rpm/
%{_libdir}/pkgconfig/rpm.pc
%{_libdir}/librp*[a-z].so
%{abidir}/rpmgraph-option.list
%{_bindir}/rpmgraph
%{_mandir}/man8/rpmgraph.8*

%files cron
%{_sysconfdir}/cron.daily/rpm
%config(noreplace) %{_sysconfdir}/logrotate.d/rpm

%files apidocs
%license COPYING
%doc docs/librpm/html/*

%changelog
* Fri Aug 25 2023 Funda Wang <fundawang@yeah.net> - 4.18.1-2
- Deal segfault with sqlite >= 3.42.0

* Wed Jul 12 2023 Funda Wang <fundawang@yeah.net> - 4.18.1-1
- New version 4.18.1

* Thu Apr 13 2023 Zhongling <zhonglingh@linux.alibaba.com> - 4.18.0-7
- Refactor rpm spec

* Thu Mar 30 2023 mgb01105731 <mgb01105731@alibaba-inc.com> - 4.18.0-6
- Del Specific Fields

* Tue Feb 28 2023 Funda Wang <fundawang@yeah.net> - 4.18.0-5
- Apply scripts used in system-rpm-config before executing configure

* Wed Nov 30 2022 chen_yu_ao <m15927509890@163.com> - 4.18.0-4
- re-add abi files that was removed before 

* Mon Nov 28 2022 chen_yu_ao <m15927509890@163.com> - 4.18.0-3
- drop the temporary build-dependency pandoc and optimise spec format

* Thu Oct 13 2022 mgb01105731 <mgb01105731@alibaba-inc.com> - 4.18.0-2
- optimise spec file

* Tue Sep 27 2022 mgb01105731 <mgb01105731@alibaba-inc.com> - 4.18.0-1
- update to 4.18.0

* Tue Mar 22 2022 Chunmei Xu <xuchunmei@linux.alibaba.com> - 4.17.0-4
- disable fsverity

* Wed Mar 09 2022 Chunmei Xu <xuchunmei@linux.alibaba.com> - 4.17.0-3
- Rebuild with new python macros

* Wed Mar 09 2022 Chunmei Xu <xuchunmei@linux.alibaba.com> - 4.17.0-2
- Bump version to rebuild

* Mon Feb 28 2022 Chunmei Xu <xuchunmei@linux.alibaba.com> - 4.17.0-1
- upstream version 4.17.0 and migrate rpmdb to /usr/lib/sysimage/rpm
